//(c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/Util",
	"cloudera/chart/AnyChart",
	"cloudera/form/TreeTableSelector"
], function(Util, AnyChart, TreeTableSelector) {

return function(options) {
	// Yes, this is stupid. But we are the victim of http://bugs.jquery.com/ticket/9134
	// (and http://bugs.jquery.com/ticket/3105 and http://bugs.jquery.com/ticket/11227).
	// When the EventSearchWidget's pane is animated shut, jQuery eventually calls domManip
	// on the whole HTML block which re-executes any script tags thus included via
	// jQuery.globalEval. I know, right?
	// For now, initialize something in the global namespace that this file will check
	// for when running. If the value is already present, early exit.
	if (window.eventSearch) {
		return;
	}
	window.eventSearch = true;
//	--------------------------------------------------------------------------
//	jQuery functions
//	--------------------------------------------------------------------------
	var treeTableSelector = null;
	var searchOnLoad = options.searchOnLoad;

	function getFormContent() {
		if (getTypesAndCodesElem().length > 0) {
			setFormElementsFromTreeTableSelection();
		}

		var content = jQuery('#eventSearchForm').serializeArray();
		var filteredContent = [];
		var i;
		for (i = 0; i < content.length; i++) {
			if (content[i].value) {
				filteredContent.push(content[i]);
			}
		}

		jQuery("input.services").not(":checked").each(function(index, item) {
			var $item = jQuery(item);
			filteredContent.push({
				name: $item.attr("name"),
				value: "off"
			});
			$item = null;
		});

		content = null;
		return jQuery.param(filteredContent);
	}

	function getTypesAndCodesElem() {
		return jQuery("#eventTypesAndCodes");
	}

	function setFormElementsFromTreeTableSelection() {
		var $typesAndCodesElem = getTypesAndCodesElem();

		var logMessageVal = treeTableSelector.hasCheckedDescendants("type_LOG_MESSAGE") ? "on" : "";
		$typesAndCodesElem.find("input[name='category_LOG_MESSAGE']").val(logMessageVal);

		var healthCheckVal = treeTableSelector.hasCheckedDescendants("type_HEALTH_CHECK") ? "on" : "";
		$typesAndCodesElem.find("input[name='category_HEALTH_CHECK']").val(healthCheckVal);

		var auditEventVal = treeTableSelector.hasCheckedDescendants("type_AUDIT_EVENT") ? "on" : "";
		$typesAndCodesElem.find("input[name='category_AUDIT_EVENT']").val(auditEventVal);

		var activityEventVal = treeTableSelector.hasCheckedDescendants("type_ACTIVITY_EVENT") ? "on" : "";
		$typesAndCodesElem.find("input[name='category_ACTIVITY_EVENT']").val(activityEventVal);

		$typesAndCodesElem = null;
	}

	function pointSelected (e) {
		var type = e.data.Series.Name;
		var value = e.data.Attributes.paramName;
		var $form = jQuery("#eventSearchForm");			
		if (type === "type") {
			// A note on the HTML structure:
			// The Event category is as such: 
			// <tr id="type_SOMEVALUE">
			//     <input type='checkbox' class='types' name=...></input>
			//     <input type='hidden' name="category_SOMEVALUE"></input>
			// </tr>
			// And then each of its child Event codes are:
			// <tr class="child-of-type_SOMEVALUE">...</tr>
			$form
				.find("input.types")
					.each(function (index, item) {
						var $item = jQuery(item);
						$item.removeAttr('checked');
						treeTableSelector.recursivelyCheckTreeTableChildren(
							$item
								.closest("tr")
									.attr("id"),
							false);
						$item = null;
					})
				.end().find(".types input[type='checkbox']")
					.removeAttr('checked')
				.end().find("input[name='category_" + value + "']")
					.each(function (index, item) {
						var $item = jQuery(item);
						treeTableSelector.recursivelyCheckTreeTableChildren(
							$item
								.closest("tr")
									.find("input[type='checkbox']")
										.attr("checked", "checked")
									.end()
								.attr("id"),
							true);
						$item = null;
					});
		} else if (type === "service") {
			$form
				.find("input.services")
					.removeAttr('checked')
				.end().find(".services input[type='checkbox']")
					.removeAttr('checked')
				.end().find("input[name='service_" + value + "']")
					.attr('checked', 'checked');
		} else if (type === "severity") {
			$form
				.find("input.severity")
					.removeAttr('checked')
				.end().find("input[name='severity_" + value + "']")
					.attr('checked', 'checked');
		}
		resetPagination();
		$form
			.find("input[name='sourceIsChart']")
				.attr("checked", "checked");
		onSearch(null);
		$form = null;
	}

	function resultsRequestCompleted(data, textStatus, jqXHR) {
		var chart = "";
		jQuery("#eventSearchForm").find(".IconSpinner24x24").addClass("hidden");
		if (textStatus === "success") {
			jQuery('.EventResultsContainer').removeClass('hidden').html(Util.filterError(jQuery.trim(data)));
			if (jQuery("#eventSearchForm").find("input[name='sourceIsChart']").attr("checked")) {
				return;
			} else if (jQuery('#eventChart').length > 0) {
				var earliestTimeStamp = jQuery(".EventResultsContainer").find("input[name='filterTimeLower']").val();
				var latestTimeStamp = jQuery(".EventResultsContainer").find("input[name='filterTimeUpper']").val();
				jQuery("#eventSearchForm")
					.find("input[name='filterTimeLower']")
						.val(earliestTimeStamp)
					.end().find("input[name='filterTimeUpper']")
						.val(latestTimeStamp);
				if (jQuery('#chartXMLURL').length > 0) {
					var options = {
						xmlUrl: jQuery('#chartXMLURL').attr('href'),
						height: jQuery("#eventSearchForm").outerHeight()-3 + "px"
					};
					var id = 'eventChart';
					chart = new AnyChart(options, id);
					chart.getChart().addEventListener("pointSelect", pointSelected);
				}
			}
		} else {
			//TODO: What to do here?
			chart = "unused";
		}
	}

	function resultsRequestFailed(jqXHR, textStatus, errorThrown) {
		//TODO: Figure out what to do here
		var unused = "This is just to make JSlint shut up";
	}

	function onSearch(e) {
		var formContent = getFormContent();
		if (!(jQuery("#eventSearchForm").find("input[name='sourceIsChart']").attr("checked"))) {
			jQuery("#eventChart").find("*").remove();
		}
		jQuery("#eventSearchForm").find(".IconSpinner24x24").removeClass("hidden");
		jQuery.ajax('/cmf/events/query', {
			dataType: 'html',
			data: formContent,
			error: resultsRequestFailed,
			success: resultsRequestCompleted,
			type: 'POST'
		});
		if (e) {
			e.preventDefault();
			e.stopPropagation();
		}
		return false;
	}

	function resetPagination() {
		jQuery("#eventSearchForm")
			.find("input[name='page']")
				.val(1);
	}

	function searchClicked(e) {
		resetPagination();
		jQuery("#eventSearchStaleMessage").addClass("hidden");
		jQuery("#eventSearchForm")
			.find("input[name='sourceIsChart']")
				.removeAttr("checked")
			.end().find("input[name='filterTimeUpper']")
				.val("")
			.end().find("input[name='filterTimeLower']")
				.val("")
			.end().find("input[name='severity']")
				.val(["CRITICAL", "INFORMATIONAL", "IMPORTANT"].join(","));
		onSearch(e);
	}

	function selectAllClicked(event) {
		var $target = jQuery(event.target);
		var $otherCheckboxes = $target.closest("div").find("input[type='checkbox']").not($target);
		Util.setCheckboxState($otherCheckboxes, $target.attr("checked"));
		$target = null;
		$otherCheckboxes = null;
	}

	function tableSortClicked(event, $target) {
		var sortField = $target.find("span.sortParamSpan").text();
		var $sortFieldInput = jQuery("input.eventSortField");
		var currentSortField = $sortFieldInput.val();
		var $eventSortReverse =  jQuery("input.eventSortReverse");
		if (currentSortField === sortField) {
			// Toggle the sort
			$eventSortReverse.prop("checked", !$eventSortReverse.prop("checked"));
		} else {
			// Sort is always ascending the first time you click a header
			$eventSortReverse.removeAttr("checked");
		}
		$sortFieldInput.val(sortField);
		onSearch(event);
	}

	function tableClicked(event) {
		var $target = jQuery(event.target);
		if ($target.hasClass("eventRowExpander")) {
			var classToRemove;
			var classToAdd;
			if ($target.hasClass('ui-icon-plus')) {
				classToRemove = 'ui-icon-plus';
				classToAdd = 'ui-icon-minus';
			} else {
				classToRemove = 'ui-icon-minus';
				classToAdd = 'ui-icon-plus';
			}
			$target.removeClass(classToRemove)
				.addClass(classToAdd)
				.closest("tr")
				.next()
				.toggleClass('hidden');
		} else if ($target.is(".EventsPaginationLinks span") ||
				   $target.is(".EventsPaginationLinks text")) {
			var currentPage = jQuery(".EventResultsContainer").find("input[name='currentPage']").val();
			currentPage = parseInt(currentPage, 10);
			var newPage;
			if ($target.closest(".prevLink").length > 0) {
				newPage = currentPage - 1;
			} else {
				newPage = currentPage + 1;
			}
			jQuery("#eventSearchForm").find("input[name='page']").val(newPage);
			onSearch(event);
		} else if ($target.hasClass("exceptionExpander")) {
            event.preventDefault();
		    var exception = jQuery("div.exceptionMessage");
		    exception.toggleClass('hidden');
		} else {
			var $th = $target.closest("th");
			if ($th.hasClass("tableSorter")) {
				tableSortClicked(event, $th);
			}
		} 
		$target = null;
	}

	var checkboxClasses = ["types", "services", "severity"];
	function singleCheckboxClicked(event) {
		var $target = jQuery(event.target);
		var classOfInterest = null;
		var i;
		for (i = 0; i < checkboxClasses.length; i++) {
			if ($target.hasClass(checkboxClasses[i])) {
				classOfInterest = checkboxClasses[i];
			}
		}
		if (!classOfInterest) {
			return;
		}

		var numUncheckedInCategory = 
			$target.closest(".componentBody")
				.find("input." + classOfInterest)
				.not(":checked").length;
		var $checkbox = jQuery(".selectAll")
			.filter("." + classOfInterest)
			.find("input[type=checkbox]");
		Util.setCheckboxState($checkbox, numUncheckedInCategory === 0);
		$target = null;
		$checkbox = null;
	}

	function noParentFoundCallback(event) {
		if (jQuery(event.target).hasClass("types")) {
			singleCheckboxClicked(event);
		}
	}

	function typesCheckboxClicked(event) {
		var $target = jQuery(event.target);
		var $targetRow = $target.closest("tr");

		var targetIsChecked = false;
		if ($target.attr("checked")) {
			targetIsChecked = true;
		}

		var id = $targetRow.attr("id");
		if (id) {
			treeTableSelector.recursivelyCheckTreeTableChildren(id, targetIsChecked);
		}
		singleCheckboxClicked(event);
		$target = null;
		$targetRow = null;
	}

	function split( val ) {
		return val.split( /,\s*/ );
	}

	function extractLast( term ) {
		return split( term ).pop();
	}

		$("#eventSearchForm").find("button[type='submit']").click(searchClicked)
			.end().find(".selectAll input").click(selectAllClicked)
			.end().find("input.types").click(typesCheckboxClicked)
			.end().find("input.services").click(singleCheckboxClicked)
			.end().find("input.severity").click(singleCheckboxClicked);
		$("#eventSearchKeywords").focus();

		$('.EventResultsContainer').click(tableClicked);
		$( "#eventSearchHost" )
			// don't navigate away from the field on tab when selecting an item
			.bind( "keydown", function( event ) {
				if ( event.keyCode === $.ui.keyCode.TAB &&
						$( this ).data( "autocomplete" ).menu.active ) {
					event.preventDefault();
				}
			})
			.autocomplete({
				source: function( request, response ) {
					$.ajax({
						url: options.autocmpPath,
						dataType: "json",
						data: {
							type: "hostName",
							prefix: extractLast( request.term ),
							num: "20"
						},
						success: function( data ) {
							response( $.map( data.suggestions, function( item ) {
								return {
									label: item,
									value: item
								};
							}));
						}
					});
				},
				search: function() {
					//custom minLength
					var term = extractLast( this.value );
					if ( term.length < 2 ) {
						return false;
					}
				},
				focus: function() {
					//prevent value inserted on focus
					return false;
				},
				select: function( event, ui ) {
					var terms = split( this.value );
					//remove the current input
					terms.pop();
					//add the selected item
					terms.push( ui.item.value );
					//add placeholder to get the comma-and-space at the end
					terms.push( "" );
					this.value = terms.join( ", " );
					return false;
				}
			});

		if (getTypesAndCodesElem().length > 0) {
			treeTableSelector = new TreeTableSelector({ 
				'rootID' : "#eventTypesAndCodes", 
				'noParentFoundCallback' : noParentFoundCallback, 
				'negativeSelector' : "input.types"
			});
		}

	function onTimeRangeChanged(range, isCurrentMode) {
		Util.text('eventSearchStartTime', range.startDate.getTime());
		Util.text("eventSearchEndTime", range.endDate.getTime());
		if (jQuery(".EventResultsContainer").hasClass("hidden")) {
			jQuery("#eventSearchStaleMessage").addClass("hidden");
		} else {
			jQuery("#eventSearchStaleMessage").removeClass("hidden");
		}
	}

	function onExpanderClicked(e) {
		// See OPSAPS-7313. IE8 has an issue when expanding the tree table
		// that prevents the #eventControlsContainer from redrawing itself
		// when individual rows are expanded. The consequence of that is an
		// ugly-looking bit where the contents of the table blow out the
		// bottom of the container without expanding the rest of it.
		// Toggling the meaningless class pokes IE8 in the eye and tells it
		// to relayout the div.
		jQuery("#eventControlsContainer").toggleClass("meaningless");
	}

	/**
	 * Handler when an event marker is selected from the chart.
	 *
	 * AnyChart allows us to select a single marker, or a marker that has been
	 * merged (on the client). This is why the input is a list.
	 * For our purposes, we only want the first event marker.
	 */
	function onEventMarkerSelected(eventMarkers) {
		if (!eventMarkers || eventMarkers.length === 0) {
			return;
		}
		var firstEventMarker = eventMarkers[0];
		jQuery('#eventSearchStartTime').val(firstEventMarker.customAttributes.timeRangeLower);
		jQuery('#eventSearchEndTime').val(firstEventMarker.customAttributes.timeRangeUpper);
		jQuery('#eventSearchCategory').val(firstEventMarker.groupId);
		jQuery("#eventSearchForm").find("button[type='submit']").trigger("click");
	}

	// This has to be on jQuery ready because we synchronize the server/client
	// clock on jQuery ready.
	onTimeRangeChanged(options);
	jQuery.subscribe("timeSelectionChanged", onTimeRangeChanged);
	jQuery.subscribe("eventMarkerSelected", onEventMarkerSelected);
	jQuery('.expander').click(onExpanderClicked);

	if (searchOnLoad) {
		onSearch(null);
	}
};
});
